# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

index_request_query = {
    'type': 'object',
    'properties': {},
    # TODO(stephenfin): Exclude additional query string parameters in a future
    # microversion
    'additionalProperties': True,
}

index_response_body = {
    'type': 'object',
    'properties': {
        'availability_zones': {
            'type': 'array',
            'items': {
                'type': 'object',
                'properties': {
                    'id': {
                        'type': 'string',
                        'format': 'uuid',
                    },
                    'name': {
                        'type': 'string',
                        'minLength': 1,
                        'maxLength': 255,
                    },
                    'created_at': {
                        'type': 'string',
                        'format': 'date-time',
                    },
                    'updated_at': {
                        'type': ['string', 'null'],
                        'format': 'date-time',
                    },
                },
                'required': [],
                'additionalProperties': False,
            },
        },
    },
    'required': ['availability_zones'],
    'additionalProperties': False,
}
